/*
 * (c) 2005 by Simon Gander <simongander@gmx.ch>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package guettingen;

import java.applet.Applet;
import java.applet.AudioClip;

/**
 * Klasse fr die Simulation eines Nachbarbahnhofes.
 * @author Simon Gander
 */

class Bahnhof implements Runnable {
	/**
	 * Konstruktor fr die Bahnhof-Klasse
	 * @param bk Zugehriger Streckenblock
	 */
	public Bahnhof (Block bk) {
		block = bk;
	}
	
	/**
	 * Zug absenden
	 * @param l Lnge des Zuges
	 */
	public void sendeZug (int l) {
		zl = l;
		
		if (lock) return;
		if (block.anfangsfeld || block.endfeld) return;
		new Thread(this, "Bahnhof").start();
	}
	
	/**
	 * Thread, der fr das Anfordern freier Bahn und das Senden des Zuges zustndig ist.
	 */
	public void run() {
		Zug z;
		
		if (!block.freieBahnAngefordert()) return;

		lock = true;
		
		while (block.freie_bahn_vorhanden) {
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				System.out.println ("Bahnhof interrupt");
			}
			if (block.freie_bahn_vorhanden && !block.freie_bahn_verlangt) {
				lock = false;
				return;
			}
		}

		try {
			Thread.sleep(2000);
		} catch (InterruptedException e) {
			System.out.println ("Bahnhof interrupt");
		}
		
		block.vorgeblockt();
		if (block.links == null) {
			z = new Zug (block, true, zl, 1500, 10);
		} else {
			z = new Zug (block, false, zl, 1500, 10);
		}

		try {
			Thread.sleep(50000);
		} catch (InterruptedException e) {
			System.out.println ("Bahnhof interrupt");
		}
		block.geblockt();
		
		try {
			Thread.sleep(3000);
		} catch (InterruptedException e) {
			System.out.println ("Bahnhof interrupt");
		}		

		if (block.links == null) zweiklang_2.play();
		else einklang_1.play();
		
		new Thread(z, "Zug").start();
		
		lock = false;
	}

	/**
	 * Verhindert, dass mehrere Threads gleichzeitig gestartet werden.
	 */
	boolean lock;
	/**
	 * Zugehriger Streckenblock
	 */
	Block block;
	/**
	 * Zuglnge
	 */
	int zl;

	/**
	 * Sound fr die Einklangglocke (1 Mal)
	 */
	static AudioClip einklang_1;
	/**
	 * Sound fr die Zweiklangglocke (2 Mal)
	 */
	static AudioClip zweiklang_2;

	static {
		einklang_1 = Applet.newAudioClip(ClassLoader.getSystemResource("sounds/EK1.wav"));
		zweiklang_2 = Applet.newAudioClip(ClassLoader.getSystemResource("sounds/ZK2.wav"));
	}
}
