/*
 * (c) 2005 by Simon Gander <simongander@gmx.ch>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package guettingen;

import java.applet.Applet;
import java.applet.AudioClip;

/**
 * Streckenblock-Steuerung.
 */
class Block extends SpurElement {
	public Block () {
		
	}
	
	public void setzeParameter (Lampe wv, Lampe rv, Lampe wn, Lampe rn, Lampe rm, Lampe shfb, Lampe af) {
		weiss_von = wv;
		rot_von = rv;
		weiss_nach = wn;
		rot_nach = rn;
		rueckmelden = rm;
		strecke_hat_freie_bahn = shfb;
		fb_angefordert = af;
		
		anfangsfeld = false;
		endfeld = false;
		freie_bahn_vorhanden = false;
		vorblock = false;
		
		lampenUpdate();
	}
	
	public void freieBahnAnfordern () {
		if (endfeld || freie_bahn_vorhanden) return;
		freie_bahn_vorhanden = true;
		blockwecker.play();
		lampenUpdate();
	}
	
	public boolean freieBahnAngefordert () {
		if (anfangsfeld || asig_zeigt_fahrt) return false;
		if (!freie_bahn_vorhanden) return true;
		if (freie_bahn_festgehalten) {
			fb_angefordert.einschalten();
			freie_bahn_verlangt = true;
			return true;
		}
		fb_angefordert.ausschalten();
		if (freie_bahn_vorhanden) blockwecker.play();
		freie_bahn_vorhanden = false;
		freie_bahn_verlangt = false;
		endfeld = false;
		lampenUpdate();
		return true;
	}

	public void freieBahnFesthalten () {
		// if (!freie_bahn_vorhanden) return;
		freie_bahn_festgehalten = !freie_bahn_festgehalten;
		if (freie_bahn_verlangt && !freie_bahn_festgehalten) {
			freieBahnAngefordert();
		}
		// An dieser Stelle k?nnte man nat?rlich noch die Taste updaten...
	}
	
	public boolean pruefeBlock() {
		if ((endfeld || anfangsfeld || !freie_bahn_vorhanden) && !block_umgehung) return false;
		return true;
	}
	
	public void blockumgehung() {
		block_umgehung = true;
	}
	
	public void asigFahrt() {
		vorblock = true;
		anfangsfeld = true;
		asig_zeigt_fahrt = true;
		freie_bahn_verlangt = false;
		fb_angefordert.ausschalten();
		lampenUpdate();
	}
	
	public void blocken() {
		vorblock = false;
		lampenUpdate();
		if (links == null) zweiklang_1.play();
		else einklang_2.play();
	}
	
	public void asigGrundstellung() {
		asig_zeigt_fahrt = false;
		vorblock = false;
		block_umgehung = false;
		if (blockwecker_ein) {
			blockwecker_ein = false;
			blockwecker_1.stop();
			blockwecker_2.play();
		}
		lampenUpdate();
	}
	
	public boolean vorgeblockt () {
		if (freie_bahn_vorhanden) return false;
		strecken_vorblock = true;
		endfeld = true;
		blockwecker.play();
		lampenUpdate();
		return true;
	}
	
	public boolean geblockt () {
		if (freie_bahn_vorhanden) return false;
		strecken_vorblock = false;
		lampenUpdate();
		return true;
	}
	
	public void rueckmeldenMoeglich () {
		if (!endfeld || freie_bahn_vorhanden) return;
		rueckmelden_moeglich = true;
		if (zug_eingefahren) {
			rueckmelden.einschalten();
		}
	}

	public void rueckmelden () {
		if (!endfeld || freie_bahn_vorhanden || !rueckmelden.ein) return;
		rueckmelden.ausschalten();
		rueckmelden_moeglich = false;
		zug_eingefahren = false;
		endfeld = false;
		lampenUpdate();
	}
	
	public void rueckgemeldet () {
		if (!anfangsfeld || !freie_bahn_vorhanden) return;
		anfangsfeld = false;
//		blockwecker.play();
		if (asig_zeigt_fahrt) {
			blockwecker_ein = true;
			blockwecker_1.loop();
		}
		lampenUpdate();
		if (freie_bahn_verlangt && !freie_bahn_festgehalten) {
			freieBahnAngefordert();
		}
	}
	
	public void lampenUpdate() {
		if (freie_bahn_vorhanden) strecke_hat_freie_bahn.ausschalten();
		else strecke_hat_freie_bahn.einschalten();

		if (anfangsfeld) rot_nach.einschalten();
		else rot_nach.ausschalten();
		if (endfeld) rot_von.einschalten();
		else rot_von.ausschalten();

		if ((!anfangsfeld || vorblock) && freie_bahn_vorhanden) weiss_nach.einschalten();
		else weiss_nach.ausschalten();
		if ((!endfeld || strecken_vorblock) && !freie_bahn_vorhanden) weiss_von.einschalten();
		else weiss_von.ausschalten();
	}
	
	public void freigeben () {
		super.freigeben();
		
		if (anfangsfeld && freie_bahn_vorhanden) {
			rueckgemeldet();
		}
	}
	
	public int signalStellung(boolean richtung) {
		if (richtung) {
			if (rechts == null) return 1;
			return rechts.signalStellung(true);
		} else {
			if (links == null) return 1;
			return links.signalStellung(false);
		}
	}

	public void zugEingefahren() {
		zug_eingefahren = true;
		if (rueckmelden_moeglich) {
			rueckmelden.einschalten();
		}
	}
	
	Lampe weiss_von;
	Lampe rot_von;
	Lampe weiss_nach;
	Lampe rot_nach;
	Lampe rueckmelden;
	Lampe strecke_hat_freie_bahn;
	Lampe fb_angefordert;
	
	// int status;	// 0 = Frei, 1 = Vorgeblockt, 2 = Geblockt
	boolean vorblock;
	boolean anfangsfeld;

	boolean strecken_vorblock;
	boolean endfeld;

	boolean freie_bahn_verlangt;
	boolean freie_bahn_vorhanden;
	boolean freie_bahn_festgehalten;

	boolean rueckmelden_moeglich;
	boolean zug_eingefahren;

	boolean block_umgehung;
	
	boolean asig_zeigt_fahrt;
	boolean blockwecker_ein;
	
	static AudioClip blockwecker;
	static AudioClip einklang_2;
	static AudioClip zweiklang_1;
	static AudioClip blockwecker_1;
	static AudioClip blockwecker_2;
	
	static {
		blockwecker = Applet.newAudioClip(ClassLoader.getSystemResource("sounds/BW.wav"));
		einklang_2 = Applet.newAudioClip(ClassLoader.getSystemResource("sounds/EK2.wav"));
		zweiklang_1 = Applet.newAudioClip(ClassLoader.getSystemResource("sounds/ZK1.wav"));
		blockwecker_1 = Applet.newAudioClip(ClassLoader.getSystemResource("sounds/BW1.wav"));
		blockwecker_2 = Applet.newAudioClip(ClassLoader.getSystemResource("sounds/BW2.wav"));
	}
}
 
