/*
 * (c) 2005 by Simon Gander <simongander@gmx.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package guettingen;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JPopupMenu;

public class Signalhebel extends Hebel implements Runnable {
	public Signalhebel(int nr, String ns, boolean fb_hebel, Signal sig, Block bk, boolean asig) {
		super(nr, ns);
		hebel_plus = sh_plus;
		hebel_minus = sh_minus;
		hat_fb_hebel = fb_hebel;
		block = bk;
		ist_asig = asig;
		signal = sig;
		
		if (ist_asig) {
			hebelsperre = asig_hsp;
		} else {
			hebelsperre = esig_hsp;
		}
		
		hsp_menu = new JPopupMenu();
		hsp_menu.add("Signalfreigabe").addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				taste = SFR_T;
				pruefeHebelsperre();
			}
		});
		hsp_menu.addSeparator();
		hsp_menu.add("Isolierumgehung fr Signalfahrtstellung").addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				taste = SIU_T;
				pruefeHebelsperre();
			}
		});
		if (ist_asig) {
			hsp_menu.add("Blockumgehung").addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					taste = BLU_T;
					pruefeHebelsperre();
				}
			});
		}
	}
	
	public void setASig(Signal s) {
		ausfahrsignal = s;
	}
	
	public void setBlockIso(SpurElement s) {
		block_iso = s;
	}
	
	public void pruefeHebelsperre() {
		boolean res = true;
		if (position) {
			taste = 0;
			return;
		}
		if (akt_fs == null) res = false;
		if (wiederholsperre) res = false;
		if (!ist_asig) {
			if (ausfahrsignal.stellung != 0) res = false;
		}
		if (taste != SIU_T) res = res && akt_fs.pruefeIsolierungen();
		if ((taste != BLU_T) && (ist_asig)) res = res && block.pruefeBlock();
		res = res && akt_fs.pruefeUeberwachung();
		if (res == true) hebelsperreEinschalten();
		else hebelsperreAusschalten();
		taste = 0;
	}
	
	public void ueberwachen() {
		if (akt_fs == null) return;
		if (!akt_fs.pruefeUeberwachung()) signalHalt();
		if (block_iso.istBelegt()) {
			if (ist_asig) {
				if (block.vorblock) block.blocken();
			} else {
				block_iso_war_belegt = true;
			}
		} else {
			if (!ist_asig) {
				if (block_iso_war_belegt && block.endfeld) {
					block.zugEingefahren();
					zug_eingefahren = true;
					repaint();
					block_iso_war_belegt = false;
				}
			}
		}
	}
	
	public void drawLampe(Graphics g) {
		if (hebelsperre_frei) g.setColor(Color.GREEN);
		else g.setColor(Color.GRAY);
		g.drawLine(28, 22, 31, 22);
		g.drawLine(27, 23, 32, 23);
		g.drawLine(26, 24, 33, 24);
		g.drawLine(26, 25, 33, 25);
		g.drawLine(26, 26, 33, 26);
		g.drawLine(26, 27, 33, 27);
		g.drawLine(27, 28, 32, 28);
		g.drawLine(28, 29, 31, 29);
		
		if (!ist_asig) {
			if (zug_eingefahren) g.setColor(Color.YELLOW);
			else g.setColor(Color.GRAY);
			g.drawLine(28, 48, 31, 48);
			g.drawLine(27, 49, 32, 49);
			g.drawLine(26, 50, 33, 50);
			g.drawLine(26, 51, 33, 51);
			g.drawLine(26, 52, 33, 52);
			g.drawLine(26, 53, 33, 53);
			g.drawLine(27, 54, 32, 54);
			g.drawLine(28, 55, 31, 55);
		}
	}
		
	public void positionMinus() {
		boolean res;
		if (!hebelsperre_frei || akt_fs == null || wiederholsperre) return;
		if (hat_fb_hebel) {
			if (akt_begriff == 1 && fb_hebel_pos == true) return;
			if (akt_begriff == 2 && fb_hebel_pos == false) return;
		}
		position = true;
		signalhebel_sound.play();
		thread_job = 1;
		new Thread(this, "SignalhebelMinus").start();
	}
	
	public void positionPlus() {
		if (festgehalten) return;
		position = false;
		if (signal.stellung != 0 && ist_asig) block.blocken();
		signal.Halt();
		thread_job = 2;
		new Thread(this, "SignalhebelPlus").start();
		signalhebel_sound.play();
	}
	
	public void signalFreigeben(int begriff, Fahrstrasse fs) {
		akt_fs = fs;
		akt_begriff = begriff;
		if (akt_fs == null) {
			wiederholsperre = false;
			hebelsperreAusschalten();
			block_iso_war_belegt = false;
		}
	}
	
	public void signalhebelFesthalten(boolean f) {
		festgehalten = f;
	}
	
	public void run() {
		switch (thread_job) {
			case 1:
				try {
					Thread.sleep(2500);
				} catch (InterruptedException e) {
					System.err.println("Interrupted Exception");
				}
				if (akt_begriff == 1) signal.Fahrt();
				if (akt_begriff == 2) signal.Fahrt2();
				wiederholsperre = true;
				if (ist_asig) block.asigFahrt();
				break;
			case 2:
				try {
					Thread.sleep(3000);
				} catch (InterruptedException e) {
					System.err.println("InterruptedException");
				}
				hebelsperreAusschalten();
				if (ist_asig) block.asigGrundstellung();
				else {
					if (zug_eingefahren) {
						block.rueckmeldenMoeglich();
						zug_eingefahren = false;
						repaint();
					}
				}
		}
	}
	
	public void signalHalt() {
		if (ist_asig && block.vorblock) block.blocken();
		signal.Halt();
	}
	
	int taste;
	static final int SFR_T = 1;
	static final int SIU_T = 2;
	static final int BLU_T = 3;
	
	private boolean zug_eingefahren;
	
	Fahrstrasse akt_fs;
	int akt_begriff;
	
	int thread_job;
	
	boolean ist_asig;
	Block block;
	Signal signal;
	Signal ausfahrsignal;
	
	boolean wiederholsperre;
	
	boolean festgehalten;
	
	SpurElement block_iso;
	boolean block_iso_war_belegt;
}
